
#ifndef _MV_CAMERA_CTRL_H_
#define _MV_CAMERA_CTRL_H_

#include "MvErrorDefine.h"
#include "CameraParams.h"
#include "MvObsoleteInterfaces.h"

/************************************************************************/
/* ̬⵼뵼                                                   */
/* Import and export definition of the dynamic library                  */
/************************************************************************/
#ifndef MV_CAMCTRL_API

    #if (defined (_WIN32) || defined(WIN64))
        #if defined(MV_CAMCTRL_EXPORTS)
            #define MV_CAMCTRL_API __declspec(dllexport)
        #else
            #define MV_CAMCTRL_API __declspec(dllimport)
        #endif
    #else
        #ifndef __stdcall
            #define __stdcall
        #endif

        #ifndef MV_CAMCTRL_API
            #define  MV_CAMCTRL_API
        #endif
    #endif

#endif

#ifndef IN
    #define IN
#endif

#ifndef OUT
    #define OUT
#endif

#ifdef __cplusplus
extern "C" {
#endif 

/************************************************************************/
/* 豸ĻָͲ                                                 */
/* Camera basic instructions and operations                             */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  ȡSDK汾
 *  @return 4ֽڰ汾
            |    |    |  |  |
             8bits  8bits  8bits  8bits
 *  @remarks 緵ֵΪ0x01000001SDK汾ΪV1.0.0.1
 
 *  @~english
 *  @brief  Get SDK Version
 *  @return Always return 4 Bytes of version number 
            |Main    |Sub    |Rev  |  Test|
             8bits  8bits  8bits  8bits
 *  @remarks For example, if the return value is 0x01000001, the SDK version is V1.0.0.1.
 ************************************************************************/
MV_CAMCTRL_API unsigned int __stdcall MV_CC_GetSDKVersion();

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡֵ֧Ĵ
 *  @return ֵ֧Ĵ 

 *  @~english
 *  @brief  Get supported Transport Layer
 *  @return Supported Transport Layer number
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_EnumerateTls();

/********************************************************************//**
 *  @~chinese
 *  @brief  ö豸
 *  @param  nTLayerType                 [IN]            öٴ
 *  @param  pstDevList                  [OUT]           豸б
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸бڴSDKڲģ̵߳øýӿʱ豸бڴͷź룬\n
             龡߳öٲ

 *  @~english
 *  @brief  Enumerate Device
 *  @param  nTLayerType                 [IN]            Enumerate TLs
 *  @param  pstDevList                  [OUT]           Device List
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks The memory of the device list is allocated within the SDK. When the interface is invoked by multiple threads, the memory of the device list will be released and applied.\n
             It is recommended to avoid multithreaded enumeration operations as much as possible.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_EnumDevices(IN unsigned int nTLayerType, IN OUT MV_CC_DEVICE_INFO_LIST* pstDevList);

/********************************************************************//**
 *  @~chinese
 *  @brief  ݳö豸
 *  @param  nTLayerType                 [IN]            öٴ
 *  @param  pstDevList                  [OUT]           豸б
 *  @param  strManufacturerName           [IN]            
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸бڴSDKڲģ̵߳øýӿʱ豸бڴͷź룬\n
             龡߳öٲ

 *  @~english
 *  @brief  Enumerate device according to manufacture name
 *  @param  nTLayerType                 [IN]            Transmission layer of enumeration
 *  @param  pstDevList                  [OUT]           Device list
 *  @param  strManufacturerName           [IN]            Manufacture Name
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks The memory of the device list is allocated within the SDK. When the interface is invoked by multiple threads, the memory of the device list will be released and applied.\n
             It is recommended to avoid multithreaded enumeration operations as much as possible.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_EnumDevicesEx(IN unsigned int nTLayerType, IN OUT MV_CC_DEVICE_INFO_LIST* pstDevList, IN const char* strManufacturerName);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸Ƿɴ
 *  @param  pstDevInfo                  [IN]            豸Ϣṹ
 *  @param  nAccessMode                 [IN]            Ȩ
 *  @return ɴtrueɴfalse
 *  @remarks ȡ豸CCPĴֵжϵǰ״̬ǷĳַȨޡ \n
             豸֧MV_ACCESS_ExclusiveWithSwitchMV_ACCESS_ControlWithSwitchMV_ACCESS_ControlSwitchEnableWithKeyģʽӿڷfalseĿǰ豸֧3ռģʽĳ̵豸Ҳݲ֧3ģʽ \n
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Is the device accessible
 *  @param  pstDevInfo                  [IN]            Device Information Structure
 *  @param  nAccessMode                 [IN]            Access Right
 *  @return Access, return true. Not access, return false
 *  @remarks Read device CCP register value and determine current access permission.\n 
             Return false if the device does not support the modes MV_ACCESS_ExclusiveWithSwitch, MV_ACCESS_ControlWithSwitch, MV_ACCESS_ControlSwitchEnableWithKey. Currently the device does not support the 3 preemption modes, neither do the devices from other mainstream manufacturers. \n
             This API is not supported by CameraLink device. 
 ************************************************************************/
MV_CAMCTRL_API bool __stdcall MV_CC_IsDeviceAccessible(IN MV_CC_DEVICE_INFO* pstDevInfo, IN unsigned int nAccessMode);

/********************************************************************//**
 *  @~chinese
 *  @brief  SDK־·
 *  @param  strSDKLogPath             [IN]           SDK־·
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ·֮󣬿ָ·־\n
             v2.4.1汾־񣬿֮ýӿЧĬ־Ϊ״̬
 
 *  @~english
 *  @brief  Set SDK log path
 *  @param  strSDKLogPath             [IN]           SDK log path
 *  @return Access, return true. Not access, return false
 *  @remarks For version V2.4.1, added log service, this API is invalid when the service is enabled.And The logging service is enabled by default\n
             This API is used to set the log file storing path.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetSDKLogPath(IN const char * strSDKLogPath);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸
 *  @param  handle                      [OUT]           豸
 *  @param  pstDevInfo                  [IN]            豸Ϣṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸ϢڲԴͳʼڲģ顣ͨýӿڴSDKӿڣĬSDK־ļҪ־ļͨMV_CC_CreateHandleWithoutLog
 
 *  @~english
 *  @brief  Create Device Handle
 *  @param  handle                      [OUT]           Device handle
 *  @param  pstDevInfo                  [IN]            Device Information Structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Create required resources within library and initialize internal module according to input device information. Create handle and call SDK interface through this interface, and SDK log file will be created by default. Creating handle through MV_CC_CreateHandleWithoutLog will not generate log files.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_CreateHandle(OUT void ** handle, IN const MV_CC_DEVICE_INFO* pstDevInfo);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸־
 *  @param  handle                      [OUT]           豸
 *  @param  pstDevInfo                  [IN]            豸Ϣṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸ϢڲԴͳʼڲģ顣ͨýӿڴSDKӿڣĬSDK־ļҪ־ļͨMV_CC_CreateHandle־ļԶɡ
 
 *  @~english
 *  @brief  Create Device Handle without log
 *  @param  handle                      [OUT]           Device handle
 *  @param  pstDevInfo                  [IN]            Device Information Structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Create required resources within library and initialize internal module according to input device information. Create handle and call SDK interface through this interface, and SDK log file will not be created. To create logs, create handle through MV_CC_CreateHandle, and log files will be automatically generated.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_CreateHandleWithoutLog(OUT void ** handle, IN const MV_CC_DEVICE_INFO* pstDevInfo);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸
 *  @param  handle                      [IN]            豸
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Destroy Device Handle
 *  @param  handle                      [IN]            Device handle
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_DestroyHandle(IN void * handle);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸
 *  @param  handle                      [IN]            豸
 *  @param  nAccessMode                 [IN]            Ȩ
 *  @param  nSwitchoverKey              [IN]            лȨʱԿ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks õ豸ҵӦ豸豸\n 
             ýӿʱɲnAccessModenSwitchoverKeyʱĬ豸ģʽΪռȨޡĿǰ豸ݲ֧MV_ACCESS_ExclusiveWithSwitchMV_ACCESS_ControlWithSwitchMV_ACCESS_ControlSwitchEnableMV_ACCESS_ControlSwitchEnableWithKeyռģʽ\n 
             U3V豸nAccessModenSwitchoverKeyЧ 
 
 *  @~english
 *  @brief  Open Device
 *  @param  handle                      [IN]            Device handle
 *  @param  nAccessMode                 [IN]            Access Right
 *  @param  nSwitchoverKey              [IN]            Switch key of access right
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Find specific device and connect according to set device parameters. \n
             When calling the interface, the input of nAccessMode and nSwitchoverKey is optional, and the device access mode is exclusive. Currently the device does not support the following preemption modes: MV_ACCESS_ExclusiveWithSwitch, MV_ACCESS_ControlWithSwitch, MV_ACCESS_ControlSwitchEnableWithKey.\n 
             For USB3Vision device, nAccessMode, nSwitchoverKey are invalid. 
 ************************************************************************/
#ifndef __cplusplus
MV_CAMCTRL_API int __stdcall MV_CC_OpenDevice(IN void* handle, IN unsigned int nAccessMode, IN unsigned short nSwitchoverKey);
#else
MV_CAMCTRL_API int __stdcall MV_CC_OpenDevice(IN void* handle, IN unsigned int nAccessMode = MV_ACCESS_Exclusive, IN unsigned short nSwitchoverKey = 0);
#endif

/********************************************************************//**
 *  @~chinese
 *  @brief  ر豸
 *  @param  handle                      [IN]            豸
 *  @return ɹMV_OK󣬷ش
 *  @remarks ͨMV_CC_OpenDevice豸󣬿ͨýӿڶϿ豸ӣͷԴ
 
 *  @~english
 *  @brief  Close Device
 *  @param  handle                      [IN]            Device handle
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After connecting to device through MV_CC_OpenDevice, use this interface to disconnect and release resources.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_CloseDevice(IN void* handle);

/********************************************************************//**
 *  @~chinese
 *  @brief  ж豸Ƿ״̬
 *  @param  handle                      [IN]            豸
 *  @return 豸״̬trueûӻʧȥӣfalse
 
 *  @~english
 *  @brief  Is The Device Connected
 *  @param  handle                      [IN]            Device handle
 *  @return Connected, return true. Not Connected or DIsconnected, return false
 ***********************************************************************/
MV_CAMCTRL_API bool __stdcall MV_CC_IsDeviceConnected(IN void* handle);

/********************************************************************//**
 *  @~chinese
 *  @brief  עͼݻص
 *  @param  handle                      [IN]            豸
 *  @param  cbOutput                    [IN]            صָ
 *  @param  pUser                       [IN]            ûԶ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ͨýӿڿͼݻصMV_CC_CreateHandle֮󼴿ɵá \n
             ͼݲɼַʽַʽܸã\n
             ʽһMV_CC_RegisterImageCallBackExͼݻصȻMV_CC_StartGrabbingʼɼɼͼõĻصзء\n
             ʽMV_CC_StartGrabbingʼɼȻӦòѭMV_CC_GetOneFrameTimeoutȡָظʽ֡ݣȡ֡ʱϲӦóҪ֡ʿƺõøýӿڵƵʡ \n
             ýӿڲ֧CameraLink豸 
 
 *  @~english
 *  @brief  Register the image callback function
 *  @param  handle                      [IN]            Device handle
 *  @param  cbOutput                    [IN]            Callback function pointer
 *  @param  pUser                       [IN]            User defined variable
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After MV_CC_CreateHandle, call this interface to set image data callback function.\n 
             There are two available image data acquisition modes, and cannot be used together: \n
             Mode 1: Call MV_CC_RegisterImageCallBack to set image data callback function, and then callMV_CC_StartGrabbing to start acquiring. The acquired image data will return in the set callback function.\n
             Mode 2: Call MV_CC_StartGrabbing to start acquiring, and then call MV_CC_GetOneFrameTimeout repeatedly in application layer to get frame data of specified pixel format. When getting frame data, the frequency of calling this interface should be controlled by upper layer application according to frame rate. \n
             This API is not supported by CameraLink device. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_RegisterImageCallBackEx(void* handle, 
                                                         void(__stdcall* cbOutput)(unsigned char * pData, MV_FRAME_OUT_INFO_EX* pFrameInfo, void* pUser),
                                                         void* pUser);

/********************************************************************//**
 *  @~chinese
 *  @brief  עͼݻصRGB
 *  @param  handle                      [IN]            豸
 *  @param  cbOutput                    [IN]            صָ
 *  @param  pUser                       [IN]            ûԶ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ͨýӿڿͼݻصMV_CC_CreateHandle֮󼴿ɵá \n
             ͼݲɼַʽַʽܸã\n
             ʽһMV_CC_RegisterImageCallBackForRGBRGB24ʽͼݻصȻMV_CC_StartGrabbingʼɼɼͼõĻصзء\n
             ʽMV_CC_StartGrabbingʼɼȻӦòѭMV_CC_GetImageForRGBȡRGB24ʽ֡ݣȡ֡ʱϲӦóҪ֡ʿƺõøýӿڵƵʡ \n
             ýӿڲ֧CameraLink豸 
 
 *  @~english
 *  @brief  register image data callback, RGB
 *  @param  handle                      [IN]            Device handle
 *  @param  cbOutput                    [IN]            Callback function pointer
 *  @param  pUser                       [IN]            User defined variable
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Before calling this API to set image data callback function, you should call this API MV_CC_CreateHandle. \n
             There are two image acquisition modes, the two modes cannot be reused: \n
             Mode 1: Call MV_CC_RegisterImageCallBackForRGB to set RGB24 format image data callback function, and then call MV_CC_StartGrabbing to start acquisition, the collected image data will be returned in the configured callback function.\n
             Mode 2: Call MV_CC_StartGrabbing to start acquisition, and the call MV_CC_GetImageForRGB repeatedly in application layer to get frame data with RGB24 format. When getting frame data, the upper application program should control the frequency of calling this API according to frame rate. \n
             This API is not supported by CameraLink device. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_RegisterImageCallBackForRGB(void* handle, 
                                                         void(__stdcall* cbOutput)(unsigned char * pData, MV_FRAME_OUT_INFO_EX* pFrameInfo, void* pUser),
                                                         void* pUser);

/********************************************************************//**
 *  @~chinese
 *  @brief  עͼݻصBGR
 *  @param  handle                      [IN]            豸
 *  @param  cbOutput                    [IN]            صָ
 *  @param  pUser                       [IN]            ûԶ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ͨýӿڿͼݻصMV_CC_CreateHandle֮󼴿ɵá\n
             ͼݲɼַʽַʽܸã\n
             ʽһMV_CC_RegisterImageCallBackForBGRBGR24ͼݻصȻMV_CC_StartGrabbingʼɼɼͼõĻصзء\n
             ʽMV_CC_StartGrabbingʼɼȻӦòѭMV_CC_GetImageForBGRȡBGR24ʽ֡ݣȡ֡ʱϲӦóҪ֡ʿƺõøýӿڵƵʡ \n
             ýӿڲ֧CameraLink豸 
 
 *  @~english
 *  @brief  register image data callback, BGR
 *  @param  handle                      [IN]            Device handle
 *  @param  cbOutput                    [IN]            Callback function pointer
 *  @param  pUser                       [IN]            User defined variable
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Before calling this API to set image data callback function, you should call this API MV_CC_CreateHandle. \n
             There are two image acquisition modes, the two modes cannot be reused: \n
             Mode 1: Call MV_CC_RegisterImageCallBackForBGR to set RGB24 format image data callback function, and then call MV_CC_StartGrabbing to start acquisition, the collected image data will be returned in the configured callback function.\n
             Mode 2: Call MV_CC_StartGrabbing to start acquisition, and the call MV_CC_GetImageForBGR repeatedly in application layer to get frame data with BGR24 format. When getting frame data, the upper application program should control the frequency of calling this API according to frame rate.\n 
             This API is not supported by CameraLink device. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_RegisterImageCallBackForBGR(void* handle, 
                                                         void(__stdcall* cbOutput)(unsigned char * pData, MV_FRAME_OUT_INFO_EX* pFrameInfo, void* pUser),
                                                         void* pUser);

/********************************************************************//**
 *  @~chinese
 *  @brief  ʼȡ
 *  @param  handle                      [IN]            豸
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Start Grabbing
 *  @param  handle                      [IN]            Device handle
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API is not supported by CameraLink device. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_StartGrabbing(IN void* handle);

/********************************************************************//**
 *  @~chinese
 *  @brief  ֹͣȡ
 *  @param  handle                      [IN]            豸
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿڲ֧CameraLink豸 
 
 *  @~english
 *  @brief  Stop Grabbing
 *  @param  handle                      [IN]            Device handle
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API is not supported by CameraLink device. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_StopGrabbing(IN void* handle);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡһ֡RGBݣ˺Ϊѯʽȡÿεòѯڲ
            ݣȡݣݷش
 *  @param  handle                      [IN]            豸
 *  @param  pData                       [OUT]           ͼݽָ
 *  @param  nDataSize                   [IN]            ջС
 *  @param  pstFrameInfo                [OUT]           ͼϢṹ
 *  @param  nMsec                       [IN]            ȴʱʱ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ÿεøýӿڣѯڲǷݣתRGB24ʽأû򷵻ش롣ΪͼתRGB24ʽкʱԵ֡ʹʱýӿڿܻᵼ¶֡\n 
             øýӿڻȡͼ֮֡ǰҪȵMV_CC_StartGrabbingͼɼýӿΪʽȡ֡ݣϲӦóҪ֡ʣƺõøýӿڵƵʡ \n
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Get one frame of RGB data, this function is using query to get data
            query whether the internal cache has data, get data if there has, return error code if no data
 *  @param  handle                      [IN]            Device handle
 *  @param  pData                       [OUT]           Image data receiving buffer
 *  @param  nDataSize                   [IN]            Buffer size
 *  @param  pstFrameInfo                [OUT]           Image information structure
 *  @param  nMsec                       [IN]            Waiting timeout
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Each time the API is called, the internal cache is checked for data. If there is data, it will be transformed as RGB24 format for return, if there is no data, return error code. As time-consuming exists when transform the image to RGB24 format, this API may cause frame loss when the data frame rate is too high. \n
             Before calling this API to get image data frame, call MV_CC_StartGrabbing to start image acquisition. This API can get frame data actively, the upper layer program should control the frequency of calling this API according to the frame rate. \n
             This API is not supported by CameraLink device.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetImageForRGB(IN void* handle, IN OUT unsigned char * pData , IN unsigned int nDataSize, IN OUT MV_FRAME_OUT_INFO_EX* pstFrameInfo, int nMsec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡһ֡BGRݣ˺Ϊѯʽȡÿεòѯڲ
            ݣȡݣݷش
 *  @param  handle                      [IN]            豸
 *  @param  pData                       [OUT]           ͼݽָ
 *  @param  nDataSize                   [IN]            ջС
 *  @param  pstFrameInfo                [OUT]           ͼϢṹ
 *  @param  nMsec                       [IN]            ȴʱʱ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ÿεøýӿڣѯڲǷݣתBGR24ʽأû򷵻ش롣ΪͼתBGR24ʽкʱԵ֡ʹʱýӿڿܻᵼ¶֡ \n
             øýӿڻȡͼ֮֡ǰҪȵMV_CC_StartGrabbingͼɼýӿΪʽȡ֡ݣϲӦóҪ֡ʣƺõøýӿڵƵʡ\n 
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Get one frame of BGR data, this function is using query to get data
            query whether the internal cache has data, get data if there has, return error code if no data
 *  @param  handle                      [IN]            Device handle
 *  @param  pData                       [OUT]           Image data receiving buffer
 *  @param  nDataSize                   [IN]            Buffer size
 *  @param  pstFrameInfo                [OUT]           Image information structure
 *  @param  nMsec                       [IN]            Waiting timeout
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Before calling this API to set image data callback function, you should call this API MV_CC_CreateHandle. \n
             There are two image acquisition modes, the two modes cannot be reused: \n
             Mode 1: Call MV_CC_RegisterImageCallBackForBGR to set RGB24 format image data callback function, and then call MV_CC_StartGrabbing to start acquisition, the collected image data will be returned in the configured callback function.\n
             Mode 2: Call MV_CC_StartGrabbing to start acquisition, and the call MV_CC_GetImageForBGR repeatedly in application layer to get frame data with BGR24 format. When getting frame data, the upper application program should control the frequency of calling this API according to frame rate. \n
             This API is not supported by CameraLink device.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetImageForBGR(IN void* handle, IN OUT unsigned char * pData , IN unsigned int nDataSize, IN OUT MV_FRAME_OUT_INFO_EX* pstFrameInfo, int nMsec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ʹڲȡһ֡ͼƬMV_CC_Displayͬʱʹã
 *  @param  handle                      [IN]            豸
 *  @param  pstFrame                    [OUT]           ͼݺͼϢ
 *  @param  nMsec                       [IN]            ȴʱʱ䣬INFINITEʱʾ޵ȴֱյһ֡ݻֹͣȡ
 *  @return ɹMV_OK󣬷ش
 *  @remarks øýӿڻȡͼ֮֡ǰҪȵMV_CC_StartGrabbingͼɼýӿΪʽȡ֡ݣϲӦóҪ֡ʣƺõøýӿڵƵʡýӿ֧óʱʱ䣬SDKڲȴֱʱأȡƽԣʺڶƽҪϸߵĳϡ \n
             ýӿMV_CC_FreeImageBufferʹãȡݺҪMV_CC_FreeImageBufferӿڽpFrameڵָȨ޽ͷš \n
             ýӿMV_CC_GetOneFrameTimeoutȣŸߵЧʡȡķsdkڲԶģMV_CC_GetOneFrameTimeoutӿҪͻз䡣\n 
             ýӿڵMV_CC_Display޷ȡ \n
             ýӿڶU3VGIGE豸֧֡ \n
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Get a frame of an image using an internal cache
 *  @param  handle                      [IN]            Device handle
 *  @param  pstFrame                    [OUT]           Image data and image information
 *  @param  nMsec                       [IN]            Waiting timeout
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Before calling this API to get image data frame, you should call MV_CC_StartGrabbing to start image acquisition. This API can get frame data actively, the upper layer program should control the frequency of calling this API according to the frame rate. This API support setting timeout, and SDK will wait to return until data appears. This function will increase the streaming stability, which can be used in the situation with high stability requirement. \n
             This API and MV_CC_FreeImageBuffer should be called in pairs, after processing the acquired data, you should call MV_CC_FreeImageBuffer to release the data pointer permission of pFrame. \n
             This interface is more efficient than MV_CC_GetOneFrameTimeout. The allocation of the stream cache is automatically allocated within the SDK.The MV_CC_GetOneFrameTimeout interface needs to be allocated by customers themselves. \n
             This API cannot be called to stream after calling MV_CC_Display. \n
             This API is not supported by CameraLink device. \n
             This API is supported by both USB3 vision camera and GigE camera. \n
 **********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetImageBuffer(IN void* handle, OUT MV_FRAME_OUT* pstFrame, IN unsigned int nMsec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ͷͼ񻺴(˽ӿͷŲʹõͼ񻺴棬MV_CC_GetImageBufferʹ)
 *  @param  handle                      [IN]            豸
 *  @param  pstFrame                    [IN]            ͼݺͼ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿMV_CC_GetImageBufferʹãʹMV_CC_GetImageBufferӿȡͼpFrameҪMV_CC_FreeImageBufferӿڽȨͷš \n
             ýӿڶȡЧʸGetOneFrameTimeoutӿڣGetImageBufferڲFree£֧ĽڵSetImageNodeӿõĽڵȣĬϽڵ1\n 
             ýӿڶU3VGIGE豸֧֡ \n
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Free image buffer(this interface can free image buffer, used with MV_CC_GetImageBuffer)
 *  @param  handle                      [IN]            Device handle
 *  @param  pstFrame                    [IN]            Image data and image information
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API and MV_CC_GetImageBuffer should be called in pairs, before calling MV_CC_GetImageBuffer to get image data pFrame, you should call MV_CC_FreeImageBuffer to release the permission. \n
             Compared with API MV_CC_GetOneFrameTimeout, this API has higher efficiency of image acquisition. The max. number of nodes can be outputted is same as the "nNum" of API MV_CC_SetImageNodeNum, default value is 1. \n
             This API is not supported by CameraLink device. \n
             This API is supported by both USB3 vision camera and GigE camera. 
 **********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_FreeImageBuffer(IN void* handle, IN MV_FRAME_OUT* pstFrame);

/********************************************************************//**
 *  @~chinese
 *  @brief  óʱƻȡһ֡ͼƬSDKڲȴֱʱ
 *  @param  handle                      [IN]            豸
 *  @param  pData                       [OUT]           ͼݽָ
 *  @param  nDataSize                   [IN]            ջС
 *  @param  pstFrameInfo                [OUT]           ͼϢṹ
 *  @param  nMsec                       [IN]            ȴʱʱ
 *  @return ɹMV_OK󣬷ش
 *  @remarks øýӿڻȡͼ֮֡ǰҪȵMV_CC_StartGrabbingͼɼýӿΪʽȡ֡ݣϲӦóҪ֡ʣƺõøýӿڵƵʡýӿ֧óʱʱ䣬SDKڲȴֱʱأȡƽԣʺڶƽҪϸߵĳϡ\n 
             ýӿڶU3VGIGE豸֧֡\n 
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Timeout mechanism is used to get image, and the SDK waits inside until the data is returned
 *  @param  handle                      [IN]            Device handle
 *  @param  pData                       [OUT]           Image data receiving buffer
 *  @param  nDataSize                   [IN]            Buffer size
 *  @param  pstFrameInfo                [OUT]           Image information structure
 *  @param  nMsec                       [IN]            Waiting timeout
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Before calling this API to get image data frame, call MV_CC_StartGrabbing to start image acquisition. This API can get frame data actively, the upper layer program should control the frequency of calling this API according to the frame rate. This API supports setting timeout, SDK will wait to return until data appears. This function will increase the streaming stability, which can be used in the situation with high stability requirement. \n
             Both the USB3Vision and GIGE camera can support this API. \n
             This API is not supported by CameraLink device.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetOneFrameTimeout(IN void* handle, IN OUT unsigned char* pData , IN unsigned int nDataSize, IN OUT MV_FRAME_OUT_INFO_EX* pstFrameInfo, unsigned int nMsec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡݻ
 *  @param  handle                      [IN]            豸
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿûڲֹͣȡʱ򣬾вҪͼ\n
             ýӿģʽдģʽ󣬿ʷݡ
 
 *  @~english
 *  @brief  if Image buffers has retrieved the dataClear them
 *  @param  handle                      [IN]            Device handle
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface allows user to clear the unnecessary images from the buffer memory without stopping acquisition. \n
             This interface allows user to clear previous data after switching from continuous mode to trigger mode. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_ClearImageBuffer(IN void* handle);

/********************************************************************//**
 *  @~chinese
 *  @brief  ʾһ֡ͼ
 *  @param  handle                      [IN]            豸
 *  @param  pstDisplayInfo              [IN]            ͼϢ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿڶU3VGIGE豸֧֡\n 
             ýӿڲ֧CameraLink豸 
 
 *  @~english
 *  @brief  Display one frame image
 *  @param  handle                      [IN]            Device handle
 *  @param  pstDisplayInfo              [IN]            Frame Info
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API is valid for USB3Vision camera and GIGE camera. \n
             This API is not supported by CameraLink device. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_DisplayOneFrame(IN void* handle, IN MV_DISPLAY_FRAME_INFO* pstDisplayInfo);

/********************************************************************//**
 *  @~chinese
 *  @brief  SDKڲͼ񻺴ڵڵ1ץͼǰ
 *  @param  handle                      [IN]            豸
 *  @param  nNum                        [IN]            ڵ
 *  @return ɹMV_OK󣬷ش
 *  @remarks øýӿڿSDKڲͼ񻺴ڵڵMV_CC_StartGrabbingʼץͼǰá\n
             SDKĬ1ڵ㡣\n
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Set the number of the internal image cache nodes in SDK, Greater than or equal to 1, to be called before the capture
 *  @param  handle                      [IN]            Device handle
 *  @param  nNum                        [IN]            Image Node Number
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Call this interface to set the number of SDK internal image buffer nodes. The interface should be called before calling MV_CC_StartGrabbing for capturing. \n
             This API is not supported by CameraLink device. 
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetImageNodeNum(IN void* handle, unsigned int nNum);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ
 *  @param  handle                      [IN]            豸
 *  @param  enGrabStrategy              [IN]            öֵ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿڶȡԣûԸʵѡ񡣾£
             -OneByOne:Ӿɵһ֡һ֡Ĵблȡͼ񣬴豸ĬΪò
             -LatestImagesOnly:блȡµһ֡ͼͬʱб
             -LatestImages:блȡµOutputQueueSize֡ͼOutputQueueSizeΧΪ1-ImageNodeNumMV_CC_SetOutputQueueSizeӿãImageNodeNumĬΪ1MV_CC_SetImageNodeNumӿ OutputQueueSizeó1ͬLatestImagesOnlyԣOutputQueueSizeóImageNodeNumͬOneByOne
             -UpcomingImage:ڵȡӿʱбͼ񣬲ȴ豸ɵһ֡ͼ񡣸òֻ֧GigE豸֧U3V豸
 
 *  @~english
 *  @brief  Set Grab Strategy
 *  @param  handle                      [IN]            Device handle
 *  @param  enGrabStrategy              [IN]            The value of Grab Strategy
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface is set by four image acquisition approaches, the user may choose one as needed. Specific details are as followed: 
             -OneByOne:Obtain image from output cache list frame by frame in order, this function is default strategy when device is on.
             -LatestImagesOnly:Obtain the latest image from output cache list only, meanwhile clear output cache list.
             -LatestImages:Obtain the latest OutputQueueSize image from output cache list, the range of OutputQueueSize is 1-ImageNodeNum, the user may set the value of MV_CC_SetOutputQueueSizeinterface,the default value of ImageNodeNum is 1,If the user usesMV_CC_SetImageNodeNuminterface to set up OutputQueueSize,when the value of OutputQueueSize is set to be 1, the function will be same as LatestImagesOnly; if the value of OutputQueueSize is set to be ImageNodeNum, the function will be same as OneByOne.
             -UpcomingImage:Ignore all images in output cache list when calling image acuiqisiotn interface, wait the next upcoming image generated. Support for GigE camera only, not support for U3V camera.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetGrabStrategy(IN void* handle, IN MV_GRAB_STRATEGY enGrabStrategy);

/********************************************************************//**
 *  @~chinese
 *  @brief  ֻMV_GrabStrategy_LatestImages²ЧΧ1-ImageNodeNum
 *  @param  handle                      [IN]            豸
 *  @param  nOutputQueueSize            [IN]            
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿLatestImagesȡ׵ãLatestImagesͼĸȡж̬
 
 *  @~english
 *  @brief  Set The Size of Output Queue(Only work under the strategy of MV_GrabStrategy_LatestImagesrang1-ImageNodeNum)
 *  @param  handle                      [IN]            Device handle
 *  @param  nOutputQueueSize            [IN]            The Size of Output Queue
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface must be used with LatestImages Grab strategy, it is used for setting the maximum allowance queue size of the image under the LatestImages strategy. The user may change the output queue size while grabbing images.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetOutputQueueSize(IN void* handle, IN unsigned int nOutputQueueSize);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ豸Ϣȡ֮ǰ
 *  @param  handle                      [IN]            豸
 *  @param  pstDevInfo                  [IN][OUT]       ظй豸Ϣṹָ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ֧ûڴ豸ȡ豸Ϣ\n 
             豸GigE豸øýӿڴգ˲ȡеøýӿڡ
 
 *  @~english
 *  @brief  Get device information
 *  @param  handle                      [IN]            Device handle
 *  @param  pstDevInfo                  [IN][OUT]       Structure pointer of device information
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks The API support users to access device information after opening the device. \n
             If the device is a GigE camera, there is a blocking risk in calling the interface, so it is not recommended to call the interface during the fetching process. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetDeviceInfo(IN void * handle, IN OUT MV_CC_DEVICE_INFO* pstDevInfo);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ͵Ϣ
 *  @param  handle                      [IN]            豸
 *  @param  pstInfo                     [IN][OUT]       ظй豸͵Ϣṹָ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ӿҪȡϢָͣMV_ALL_MATCH_INFOṹеnTypeͣȡӦϢMV_ALL_MATCH_INFOṹpInfoﷵأ \n
             ýӿڵĵǰȡȡϢͣȡGigE豸MV_MATCH_TYPE_NET_DETECTϢڿץͼ֮ãȡU3V豸MV_MATCH_TYPE_USB_DETECTϢڴ豸֮á \n
             ýӿڲ֧CameraLink豸 
 
 *  @~english
 *  @brief  Get various type of information
 *  @param  handle                      [IN]            Device handle
 *  @param  pstInfo                     [IN][OUT]       Structure pointer of various type of information
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Input required information type (specify nType in structure MV_ALL_MATCH_INFO) in the interface and get corresponding information (return in pInfo of structure MV_ALL_MATCH_INFO). \n
             The calling precondition of this interface is determined by obtained information type. Call after enabling capture to get MV_MATCH_TYPE_NET_DETECT information of GigE device, and call after starting device to get MV_MATCH_TYPE_USB_DETECT information of USB3Vision device. \n
             This API is not supported by CameraLink device. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetAllMatchInfo(IN void* handle, IN OUT MV_ALL_MATCH_INFO* pstInfo);


/************************************************************************/
/* úͻȡ豸ܽӿ                                         */
/* General interface for getting and setting camera parameters          */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  ȡIntegerֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            ԼֵȡϢΪ"Width"
 *  @param  pstIntValue                 [IN][OUT]       ظй豸Խṹָ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿԻȡint͵ָڵֵstrKeyȡֵԲοXMLڵбΪIIntegerĽڵֵͨýӿڻȡstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Get Integer value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value, for example, using "Width" to get width
 *  @param  pstIntValue                 [IN][OUT]       Structure pointer of camera features
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks You can call this API to get the value of camera node with integer type after connecting the device. For strKey value, refer to MvCameraNode. All the node values of "IInteger" in the list can be obtained via this API. strKey corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetIntValueEx(IN void* handle,IN const char* strKey,OUT MVCC_INTVALUE_EX *pstIntValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  Integerֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            ԼֵȡϢΪ"Width"
 *  @param  nValue                      [IN]            Ҫõ豸ֵ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿint͵ָڵֵstrKeyȡֵԲοXMLڵбΪIIntegerĽڵֵͨýӿãstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Set Integer value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value, for example, using "Width" to set width
 *  @param  nValue                      [IN]            Feature value to set
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks You can call this API to get the value of camera node with integer type after connecting the device. For strKey value, refer to MvCameraNode. All the node values of "IInteger" in the list can be obtained via this API. strKey corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetIntValueEx(IN void* handle,IN const char* strKey,IN int64_t nValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡEnumֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            ԼֵȡظʽϢΪ"PixelFormat"
 *  @param  pstEnumValue                [IN][OUT]       ظй豸Խṹָ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿԻȡEnum͵ָڵֵstrKeyȡֵԲοXMLڵбΪIEnumerationĽڵֵͨýӿڻȡstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Get Enum value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value, for example, using "PixelFormat" to get pixel format
 *  @param  pstEnumValue                [IN][OUT]       Structure pointer of camera features
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to get specified Enum nodes. For value of strKey, see MvCameraNode, The node values of IEnumeration can be obtained through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetEnumValue(IN void* handle,IN const char* strKey,OUT MVCC_ENUMVALUE *pstEnumValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  Enumֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            ԼֵȡظʽϢΪ"PixelFormat"
 *  @param  nValue                      [IN]            Ҫõ豸ֵ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿEnum͵ָڵֵstrKeyȡֵԲοXMLڵбΪIEnumerationĽڵֵͨýӿãstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Set Enum value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value, for example, using "PixelFormat" to set pixel format
 *  @param  nValue                      [IN]            Feature value to set
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to get specified Enum nodes. For value of strKey, see MvCameraNode, The node values of IEnumeration can be obtained through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetEnumValue(IN void* handle,IN const char* strKey,IN unsigned int nValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  Enumֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            ԼֵȡظʽϢΪ"PixelFormat"
 *  @param  strValue                    [IN]            Ҫõ豸ַ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿEnum͵ָڵֵstrKeyȡֵԲοXMLڵбΪIEnumerationĽڵֵͨýӿãstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Set Enum value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value, for example, using "PixelFormat" to set pixel format
 *  @param  strValue                    [IN]            Feature String to set
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Call this API after connecting the device. All the values of nodes with IEnumeration type can be set via this API.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetEnumValueByString(IN void* handle,IN const char* strKey,IN const char* strValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡFloatֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            Լֵ
 *  @param  pstFloatValue               [IN][OUT]       ظй豸Խṹָ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿԻȡfloat͵ָڵֵstrKeyȡֵԲοXMLڵбΪIFloatĽڵֵͨýӿڻȡstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Get Float value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value
 *  @param  pstFloatValue               [IN][OUT]       Structure pointer of camera features
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to get specified float node. For detailed strKey value see: MvCameraNode. The node values of IFloat can be obtained through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetFloatValue(IN void* handle,IN const char* strKey,OUT MVCC_FLOATVALUE *pstFloatValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  floatֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            Լֵ
 *  @param  fValue                      [IN]            Ҫõ豸ֵ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿfloat͵ָڵֵstrKeyȡֵԲοXMLڵбΪIFloatĽڵֵͨýӿãstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Set float value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value
 *  @param  fValue                      [IN]            Feature value to set
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to set specified float node. For detailed strKey value see: MvCameraNode. The node values of IFloat can be set through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetFloatValue(IN void* handle,IN const char* strKey,IN float fValue);
    
/********************************************************************//**
 *  @~chinese
 *  @brief  ȡBooleanֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            Լֵ
 *  @param  pbValue                     [IN][OUT]       ظй豸ֵ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿԻȡbool͵ָڵֵstrKeyȡֵԲοXMLڵбΪIBooleanĽڵֵͨýӿڻȡstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Get Boolean value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value
 *  @param  pbValue                     [IN][OUT]       Structure pointer of camera features
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to get specified bool nodes. For value of strKey, see MvCameraNode. The node values of IBoolean can be obtained through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetBoolValue(IN void* handle,IN const char* strKey,OUT bool *pbValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  Booleanֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            Լֵ
 *  @param  bValue                      [IN]            Ҫõ豸ֵ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿbool͵ָڵֵstrKeyȡֵԲοXMLڵбΪIBooleanĽڵֵͨýӿãstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Set Boolean value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value
 *  @param  bValue                      [IN]            Feature value to set
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to set specified bool nodes. For value of strKey, see MvCameraNode. The node values of IBoolean can be set through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetBoolValue(IN void* handle,IN const char* strKey,IN bool bValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡStringֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            Լֵ
 *  @param  pstStringValue              [IN][OUT]       ظй豸Խṹָ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿԻȡstring͵ָڵֵstrKeyȡֵԲοXMLڵбΪIStringĽڵֵͨýӿڻȡstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Get String value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value
 *  @param  pstStringValue              [IN][OUT]       Structure pointer of camera features
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to get specified string nodes. For value of strKey, see MvCameraNode. The node values of IString can be obtained through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetStringValue(IN void* handle,IN const char* strKey,OUT MVCC_STRINGVALUE *pstStringValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  Stringֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            Լֵ
 *  @param  strValue                    [IN]            Ҫõ豸ֵ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿstring͵ָڵֵstrKeyȡֵԲοXMLڵбΪIStringĽڵֵͨýӿãstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Set String value
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value
 *  @param  strValue                    [IN]            Feature value to set
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to set specified string nodes. For value of strKey, see MvCameraNode. The node values of IString can be set through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetStringValue(IN void* handle,IN const char* strKey,IN const char* strValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  Commandֵ
 *  @param  handle                      [IN]            豸
 *  @param  strKey                      [IN]            Լֵ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸֮øýӿڿָCommandͽڵ㡣strKeyȡֵԲοXMLڵбΪICommandĽڵ㶼ͨýӿãstrKeyȡֵӦбġơһС
 
 *  @~english
 *  @brief  Send Command
 *  @param  handle                      [IN]            Device handle
 *  @param  strKey                      [IN]            Key value
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to set specified Command nodes. For value of strKey, see MvCameraNode. The node values of ICommand can be set through this interface, strKey value corresponds to the Name column.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetCommandValue(IN void* handle,IN const char* strKey);

/********************************************************************//**
 *  @~chinese
 *  @brief  GenICamڵ㻺
 *  @param  handle                      [IN]            豸
 *  @return ɹMV_OK󣬷ش
 
 *  @~english
 *  @brief  Invalidate GenICam Nodes
 *  @param  handle                      [IN]            Device handle
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_InvalidateNodes(IN void* handle);


/************************************************************************/
/* 豸  Ĵд 쳣¼ص                                */
/* Device upgrade, register read and write and exception callback       */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  豸
 *  @param  handle                      [IN]            豸
 *  @param  strFilePathName             [IN]            ļ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ͨýӿڿԽ̼ļ͸豸ýӿҪȴ̼ļɹ豸֮ٷأӦʱܽϳ
 
 *  @~english
 *  @brief  Device Local Upgrade
 *  @param  handle                      [IN]            Device handle
 *  @param  strFilePathName             [IN]            File name
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Call this API to send the upgrade firmware to the device for upgrade. This API will wait for return until the upgrade firmware is sent to the device, this response may take a long time. \n
             For CameraLink device, it keeps sending upgrade firmware continuously. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_LocalUpgrade(IN void* handle, const void* strFilePathName);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ
 *  @param  handle                      [IN]            豸
 *  @param  pnProcess                   [OUT]           Ƚյַ
 *  @return ɹ,MV_OK,ʧ,ش
 
 *  @~english
 *  @brief  Get Upgrade Progress
 *  @param  handle                      [IN]            Device handle
 *  @param  pnProcess                   [OUT]           Progress receiving address
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetUpgradeProcess(IN void* handle, unsigned int* pnProcess);

/********************************************************************//**
 *  @~chinese
 *  @brief  ڴ
 *  @param  handle                      [IN]            豸
 *  @param  pBuffer                     [IN][OUT]       ΪֵʹãڴֵGEV豸ڴֵǰմģʽ洢ģЭ豸С˴洢
 *  @param  nAddress                    [IN]            ȡڴַõַԴ豸Camera.xmlļлȡxxx_RegAddrxmlڵֵ
 *  @param  nLength                     [IN]            ȡڴ泤
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸ȡĳμĴݡ
 
 *  @~english
 *  @brief  Read Memory
 *  @param  handle                      [IN]            Device Handle
 *  @param  pBuffer                     [IN][OUT]       Used as a return value, save the read-in memory value ( Memory value is stored in accordance with the big end model)
 *  @param  nAddress                    [IN]            Memory address to be read, which can be obtained from the Camera.xml file of the device, the form xml node value of xxx_RegAddr
 *  @param  nLength                     [IN]            Length of the memory to be read
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Access device, read the data from certain register.
*************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_ReadMemory(IN void* handle , void *pBuffer, int64_t nAddress, int64_t nLength);

/********************************************************************//**
 *  @~chinese
 *  @brief  дڴ
 *  @param  handle                      [IN]            豸
 *  @param  pBuffer                     [IN]            дڴֵעGEV豸ڴֵҪմģʽ洢Э豸С˴洢
 *  @param  nAddress                    [IN]            дڴַõַԴ豸Camera.xmlļлȡxxx_RegAddrxmlڵֵ
 *  @param  nLength                     [IN]            дڴ泤
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸һдĳμĴ
 
 *  @~english
 *  @brief  Write Memory
 *  @param  handle                      [IN]            Device Handle
 *  @param  pBuffer                     [IN]            Memory value to be written ( Note the memory value to be stored in accordance with the big end model)
 *  @param  nAddress                    [IN]            Memory address to be written, which can be obtained from the Camera.xml file of the device, the form xml node value of xxx_RegAddr
 *  @param  nLength                     [IN]            Length of the memory to be written
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Access device, write a piece of data into a certain segment of register.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_WriteMemory(IN void* handle, const void *pBuffer, int64_t nAddress, int64_t nLength);

/********************************************************************//**
 *  @~chinese
 *  @brief  ע쳣Ϣصڴ豸֮
 *  @param  handle                      [IN]            豸
 *  @param  cbException                 [IN]            쳣صָ
 *  @param  pUser                       [IN]            ûԶ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ýӿҪMV_CC_OpenDevice豸֮á豸쳣ϿӺڻصȡ쳣ϢGigE豸֮ҪȵMV_CC_CloseDeviceӿڹر豸ٵMV_CC_OpenDeviceӿ´豸
 
 *  @~english
 *  @brief  Register Exception Message CallBack, call after open device
 *  @param  handle                      [IN]            Device handle
 *  @param  cbException                 [IN]            Exception Message CallBack Function Pointer
 *  @param  pUser                       [IN]            User defined variable
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Call this interface after the device is opened by MV_CC_OpenDevice. When device is exceptionally disconnected, the exception message can be obtained from callback function. For Disconnected GigE device, first call MV_CC_CloseDevice to shut device, and then call MV_CC_OpenDevice to reopen the device. 
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_RegisterExceptionCallBack(IN void* handle, 
                                                             void(__stdcall* cbException)(unsigned int nMsgType, void* pUser), void* pUser);

/********************************************************************//**
 *  @~chinese
 *  @brief  עȫ¼صڴ豸֮
 *  @param  handle                      [IN]            豸
 *  @param  cbEvent                     [IN]            ¼صָ
 *  @param  pUser                       [IN]            ûԶ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ͨýӿ¼صڻصȡɼع¼Ϣ \n
             ýӿڲ֧CameraLink豸
 
 *  @~english
 *  @brief  Register event callback, which is called after the device is opened
 *  @param  handle                      [IN]            Device handle
 *  @param  cbEvent                     [IN]            Event CallBack Function Pointer
 *  @param  pUser                       [IN]            User defined variable
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Call this API to set the event callback function to get the event information, e.g., acquisition, exposure, and so on. \n
             This API is not supported by CameraLink device.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_RegisterAllEventCallBack(void* handle, void(__stdcall* cbEvent)(MV_EVENT_OUT_INFO * pEventInfo, void* pUser), void* pUser);

/********************************************************************//**
 *  @~chinese
 *  @brief  עᵥ¼صڴ豸֮
 *  @param  handle                      [IN]            豸
 *  @param  strEventName                [IN]            ¼
 *  @param  cbEvent                     [IN]            ¼صָ
 *  @param  pUser                       [IN]            ûԶ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ͨýӿ¼صڻصȡɼع¼Ϣ\n 
             ýӿڲ֧CameraLink豸֧"豸"һ¼ 
 
 *  @~english
 *  @brief  Register single event callback, which is called after the device is opened
 *  @param  handle                      [IN]            Device handle
 *  @param  strEventName                [IN]            Event name
 *  @param  cbEvent                     [IN]            Event CallBack Function Pointer
 *  @param  pUser                       [IN]            User defined variable
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Call this API to set the event callback function to get the event information, e.g., acquisition, exposure, and so on. \n
             This API is not supported by CameraLink device .
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_RegisterEventCallBackEx(void* handle, const char* strEventName,
                                                         void(__stdcall* cbEvent)(MV_EVENT_OUT_INFO * pEventInfo, void* pUser), void* pUser);


/************************************************************************/
/* GigEVision 豸еĽӿ                                             */
/* GigEVision device specific interface                                 */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  ǿIP
 *  @param  handle                      [IN]            豸
 *  @param  nIP                         [IN]            õIP
 *  @param  nSubNetMask                 [IN]            
 *  @param  nDefaultGateWay             [IN]            Ĭ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ǿ豸IP롢Ĭأǿ֮Ҫ´豸GigEVision豸֧֡\n
             豸ΪDHCP״̬øýӿǿ豸֮豸
 
 *  @~english
 *  @brief  Force IP
 *  @param  handle                      [IN]            Device handle
 *  @param  nIP                         [IN]            IP to set
 *  @param  nSubNetMask                 [IN]            Subnet mask
 *  @param  nDefaultGateWay             [IN]            Default gateway
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Force setting camera network parameter (including IP address, subnet mask, default gateway). After forced setting, device handle should be created again. This function is only supported by GigEVision camera.\n
             If device is in DHCP status, after calling this API to force setting camera network parameter, the device will restart.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_ForceIpEx(IN void* handle, unsigned int nIP, unsigned int nSubNetMask, unsigned int nDefaultGateWay);

/********************************************************************//**
 *  @~chinese
 *  @brief  IPʽ
 *  @param  handle                      [IN]            豸
 *  @param  nType                       [IN]            IPͣMV_IP_CFG_x
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks 豸IPʽDHCPLLAȣGigEVision豸֧֡
 
 *  @~english
 *  @brief  IP configuration method
 *  @param  handle                      [IN]            Device handle
 *  @param  nType                       [IN]            IP type, refer to MV_IP_CFG_x
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Send command to set camera IP mode, such as DHCP and LLA, only supported by GigEVision.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_SetIpConfig(IN void* handle, unsigned int nType);

/********************************************************************//**
 *  @~chinese
 *  @brief  ýʹĳģʽ,type: MV_NET_TRANS_xʱĬʹdriver
 *  @param  handle                      [IN]            豸
 *  @param  nType                       [IN]            紫ģʽMV_NET_TRANS_x
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ͨýӿڿSDKڲʹõģʽĬʹģʽGigEVision豸֧֡
 
 *  @~english
 *  @brief  Set to use only one mode,type: MV_NET_TRANS_x. When do not set, priority is to use driver by default
 *  @param  handle                      [IN]            Device handle
 *  @param  nType                       [IN]            Net transmission mode, refer to MV_NET_TRANS_x
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarksSet SDK internal priority network mode through this interface, drive mode by default, only supported by GigEVision camera.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_SetNetTransMode(IN void* handle, unsigned int nType);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ紫Ϣ
 *  @param  handle                      [IN]            豸
 *  @param  pstInfo                     [OUT]           Ϣṹ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ͨýӿڿԻȡ紫ϢѽݴС֡ȣMV_CC_StartGrabbingɼ֮áGigEVision豸֧֡
 
 *  @~english
 *  @brief  Get net transmission information
 *  @param  handle                      [IN]            Device handle
 *  @param  pstInfo                     [OUT]           Information Structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Get network transmission information through this API, including received data size, number of lost frames. Call this API after starting image acquiring through MV_CC_StartGrabbing. This API is supported only by GigEVision Camera.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_GetNetTransInfo(IN void* handle, MV_NETTRANS_INFO* pstInfo);

/********************************************************************//**
 *  @~chinese
 *  @brief  öĻظ
 *  @param  nMode                       [IN]            ظͣĬϹ㲥0-1-㲥
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿֻGigEЧ

 *  @~english
 *  @brief  Setting the ACK mode of devices Discovery.
 *  @param  nMode                       [IN]            ACK modeDefault-Broadcast,0-Unicast,1-Broadcast.
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks This interface is ONLY effective on GigE cameras. \n
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_SetDiscoveryMode(unsigned int nMode);

/********************************************************************//**
 *  @~chinese
 *  @brief  GVSPȡʱʱ
 *  @param  handle                      [IN]            豸
 *  @param  nMillisec                   [IN]            ʱʱ䣬Ĭ300msΧ>10ms
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸֮ȡǰøýӿڿGVSPȡʱʱ䡣GVSPȡʱù̿ͼ쳣ùȡʱ䳤
 
 *  @~english
 *  @brief  Set GVSP streaming timeout
 *  @param  handle                      [IN]            Device handle
 *  @param  nMillisec                   [IN]            Timeout, default 300ms, range: >10ms
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, and just before start streaming, 
 *           call this interface to set GVSP streaming timeout value.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_SetGvspTimeout(void* handle, unsigned int nMillisec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡGVSPȡʱʱ
 *  @param  handle                      [IN]            豸
 *  @param  pnMillisec                  [OUT]           ʱʱָ룬Ժλλ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿڻȡǰGVSPȡʱʱ
 
 *  @~english
 *  @brief  Get GVSP streaming timeout
 *  @param  handle                      [IN]            Device handle
 *  @param  pnMillisec                  [OUT]           Timeout, ms as unit
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface is used to get the current GVSP streaming timeout.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_GetGvspTimeout(IN void* handle, unsigned int* pnMillisec);

/********************************************************************//**
 *  @~chinese
 *  @brief  GVCPʱʱ
 *  @param  handle                      [IN]            豸
 *  @param  nMillisec                   [IN]            ʱʱ䣬Ĭ500msΧ0-10000ms
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸֮øýӿڿGVCPʱʱ䡣
 
 *  @~english
 *  @brief  Set GVCP cammand timeout
 *  @param  handle                      [IN]            Device handle
 *  @param  nMillisec                   [IN]            Timeout, default 500ms, range: 0-10000ms
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to set GVCP command timeout time.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_SetGvcpTimeout(void* handle, unsigned int nMillisec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡGVCPʱʱ
 *  @param  handle                      [IN]            豸
 *  @param  pnMillisec                  [OUT]           ʱʱָ룬Ժλλ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿڻȡǰGVCPʱʱ䡣
 
 *  @~english
 *  @brief  Get GVCP cammand timeout
 *  @param  handle                      [IN]            Device handle
 *  @param  pnMillisec                  [OUT]           Timeout, ms as unit
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface is used to get the current GVCP timeout.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_GetGvcpTimeout(IN void* handle, unsigned int* pnMillisec);

/********************************************************************//**
 *  @~chinese
 *  @brief  شGVCP
 *  @param  handle                      [IN]            豸
 *  @param  nRetryGvcpTimes             [IN]            شΧ0-100
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿGVCP쳣ʱشĴһ̶ϿԱ豸ߣΧΪ0-100
 
 *  @~english
 *  @brief  Set the number of retry GVCP cammand
 *  @param  handle                      [IN]            Device handle
 *  @param  nRetryGvcpTimes             [IN]            The number of retriesrang0-100
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface is used to increase The Times of retransmission when GVCP packet transmission is abnormal,and to some extent, it can avoid dropping the camera, with a range of 0-100.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_SetRetryGvcpTimes(IN void* handle, unsigned int nRetryGvcpTimes);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡشGVCP
 *  @param  handle                      [IN]            豸
 *  @param  pnRetryGvcpTimes            [OUT]           شָ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿڻȡǰGVCPشĬ3Ρ
 
 *  @~english
 *  @brief  Get the number of retry GVCP cammand
 *  @param  handle                      [IN]            Device handle
 *  @param  pnRetryGvcpTimes            [OUT]           The number of retries
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface is used to get the current number of GVCP retransmissions, which defaults to 3.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_GetRetryGvcpTimes(IN void* handle, unsigned int* pnRetryGvcpTimes);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡѵpacket sizeýӿĿǰֻ֧GigE豸
 *  @param  handle                      [IN]            豸
 *  @return packetsize
 *  @remarks ȡѵpacket sizeӦGigEVision豸SCPSӦU3V豸ÿδȡİСôСϴһĴСýӿҪMV_CC_OpenDevice֮MV_CC_StartGrabbing֮ǰá \n
             ýӿڲ֧CameraLink豸 
 
 *  @~english
 *  @brief  Get the optimal Packet Size, Only support GigE Camera
 *  @param  handle                      [IN]            Device handle
 *  @return Optimal packetsize
 *  @remarks To get optimized packet size, for GigEVision device is SCPS, for USB3Vision device is the size of packet read from drive each time, and it is the size of a packet transported on the network. The interface should be called after MV_CC_OpenDevice and before MV_CC_StartGrabbing. \n
             This API is not supported by CameraLink device. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetOptimalPacketSize(IN void* handle);

/********************************************************************//**
 *  @~chinese
 *  @brief  Ƿط֧֣ط
 *  @param  handle                      [IN]            豸
 *  @param  bEnable                     [IN]            Ƿ֧ط
 *  @param  nMaxResendPercent           [IN]            ط
 *  @param  nResendTimeout              [IN]            طʱʱ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸֮øýӿڿطԣGigEVision豸֧֡
 
 *  @~english
 *  @brief  Set whethe to enable resend, and set resend
 *  @param  handle                      [IN]            Device handle
 *  @param  bEnable                     [IN]            enable resend
 *  @param  nMaxResendPercent           [IN]            Max resend persent
 *  @param  nResendTimeout              [IN]            Resend timeout
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After the device is connected, call this interface to set resend packet properties, only supported by GigEVision camera.
 ************************************************************************/
#ifndef __cplusplus
MV_CAMCTRL_API int __stdcall MV_GIGE_SetResend(void* handle, unsigned int bEnable, unsigned int nMaxResendPercent, unsigned int nResendTimeout);
#else
MV_CAMCTRL_API int __stdcall MV_GIGE_SetResend(void* handle, unsigned int bEnable, unsigned int nMaxResendPercent = 10, unsigned int nResendTimeout = 50);
#endif

/********************************************************************//**
 *  @~chinese
 *  @brief  شԴ
 *  @param  handle                      [IN]            豸
 *  @param  nRetryTimes                 [IN]            شԴĬ20
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿڱڵMV_GIGE_SetResendش֮ãʧҷMV_E_CALLORDER
 
 *  @~english
 *  @brief  set the max resend retry times
 *  @param  handle                      [IN]            Device handle
 *  @param  nRetryTimes                 [IN]            The max times to retry resending lost packetsdefault 20
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface MUST be called after enabling resending lost packets by calling MV_GIGE_SetResend, \n
 *           otherwise would fail and return MV_E_CALLORDER.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall  MV_GIGE_SetResendMaxRetryTimes(void* handle, unsigned int nRetryTimes);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡشԴ
 *  @param  handle                      [IN]            豸
 *  @param  pnRetryTimes                [OUT]           شԴ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿڱڵMV_GIGE_SetResendش֮ãʧҷMV_E_CALLORDER
 
 *  @~english
 *  @brief  get the max resend retry times
 *  @param  handle                      [IN]            Device handle
 *  @param  pnRetryTimes                [OUT]           The max times to retry resending lost packets
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface MUST be called after enabling resending lost packets by calling MV_GIGE_SetResend, \n
 *           otherwise would fail and return MV_E_CALLORDER. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall  MV_GIGE_GetResendMaxRetryTimes(void* handle, unsigned int* pnRetryTimes);

/********************************************************************//**
 *  @~chinese
 *  @brief  ͬһش֮ʱ
 *  @param  handle                      [IN]            豸
 *  @param  nMillisec                   [IN]            ͬһش֮ʱĬ10ms
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿڱڵMV_GIGE_SetResendش֮ãʧҷMV_E_CALLORDER
 
 *  @~english
 *  @brief  set time interval between same resend requests
 *  @param  handle                      [IN]            Device handle
 *  @param  nMillisec                   [IN]            The time interval between same resend requests,default 10ms
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface MUST be called after enabling resending lost packets by calling MV_GIGE_SetResend, \n
 *           otherwise would fail and return MV_E_CALLORDER. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall  MV_GIGE_SetResendTimeInterval(void* handle, unsigned int nMillisec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡͬһش֮ʱ
 *  @param  handle                      [IN]            豸
 *  @param  pnMillisec                  [OUT]           ͬһش֮ʱ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿڱڵMV_GIGE_SetResendش֮ãʧҷMV_E_CALLORDER
 
 *  @~english
 *  @brief  get time interval between same resend requests
 *  @param  handle                      [IN]            Device handle
 *  @param  pnMillisec                  [OUT]           The time interval between same resend requests
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface MUST be called after enabling resending lost packets by calling MV_GIGE_SetResend, \n
 *           otherwise would fail and return MV_E_CALLORDER. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall  MV_GIGE_GetResendTimeInterval(void* handle, unsigned int* pnMillisec);

/********************************************************************//**
 *  @~chinese
 *  @brief  ôģʽΪģʽ鲥ģʽ
 *  @param  handle                      [IN]            豸
 *  @param  stTransmissionType          [IN]            ģʽṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ͨýӿڿôģʽΪ鲥ģʽGigEVision豸֧֡

 *  @~english
 *  @brief  Set transmission type,Unicast or Multicast
 *  @param  handle                      [IN]            Device handle
 *  @param  stTransmissionType          [IN]            Struct of transmission type
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Call this API to set the transmission mode as single cast mode and multicast mode. And this API is only valid for GigEVision camera. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_SetTransmissionType(void* handle, MV_TRANSMISSION_TYPE * pstTransmissionType);

/********************************************************************//**
 *  @~chinese
 *  @brief   
 *  @param   pstActionCmdInfo           [IN]            Ϣ
 *  @param   pstActionCmdResults        [OUT]           Ϣб
 *  @return ɹMV_OK󣬷ش 
 *  @remarks GigEVision豸֧֡

 *  @~english
 *  @brief  Issue Action Command
 *  @param   pstActionCmdInfo           [IN]            Action Command
 *  @param   pstActionCmdResults        [OUT]           Action Command Result List
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API is supported only by GigEVision camera.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_IssueActionCommand(IN MV_ACTION_CMD_INFO* pstActionCmdInfo, OUT MV_ACTION_CMD_RESULT_LIST* pstActionCmdResults);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ鲥״̬
 *  @param  pstDevInfo                  [IN]            豸Ϣṹ
 *  @param  pbStatus                    [OUT]           鲥״̬,true:鲥״̬false:鲥
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿж豸ǰǷ鲥״̬ͻöʱҪ豸ж鲥⡣

 *  @~english
 *  @brief  Get Multicast Status
 *  @param  pstDevInfo                  [IN]            Device Information Structure
 *  @param  pbStatus                    [OUT]           Status of Multicast
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This interface is used to determine whether the camera is currently in multicast state, and to solve the problem that the client needs to turn on the camera to determine multicast when enumerating.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_GIGE_GetMulticastStatus(IN MV_CC_DEVICE_INFO* pstDevInfo, OUT bool* pbStatus);


/************************************************************************/
/* CameraLink 豸еĽӿ                                            */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  豸
 *  @param  handle                      [IN]            豸
 *  @param  nBaudrate                   [IN]            õĲֵֵοCameraParams.hк궨壬#define MV_CAML_BAUDRATE_9600  0x00000001
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ýӿ֧豸δʱá
 
 *  @~english
 *  @brief  Set device baudrate using one of the CL_BAUDRATE_XXXX value
 *  @param  handle                      [IN]            Device handle
 *  @param  nBaudrate                   [IN]            baud rate to set. Refer to the 'CameraParams.h' for parameter definitions, for example, #define MV_CAML_BAUDRATE_9600  0x00000001
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks This API is supported only by CameraLink device.\n
             This API support calls when devices are not connected.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CAML_SetDeviceBaudrate(IN void* handle, unsigned int nBaudrate);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ豸
 *  @param  handle                      [IN]            豸
 *  @param  pnCurrentBaudrate           [OUT]           Ϣָ룬ֵοCameraParams.hк궨壬#define MV_CAML_BAUDRATE_9600  0x00000001
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ýӿ֧豸δʱá
 
 *  @~english
 *  @brief  Returns the current device baudrate, using one of the CL_BAUDRATE_XXXX value
 *  @param  handle                      [IN]            Device handle
 *  @param  pnCurrentBaudrate           [OUT]           Return pointer of baud rate to user. Refer to the 'CameraParams.h' for parameter definitions, for example, #define MV_CAML_BAUDRATE_9600  0x00000001
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks This API is supported only by CameraLink device.\n
             This API support calls when devices are not connected.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CAML_GetDeviceBaudrate(IN void* handle,unsigned int* pnCurrentBaudrate);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ豸ֵ֧Ĳ
 *  @param  handle                      [IN]            豸
 *  @param  pnBaudrateAblity            [OUT]           ֵ֧ĲϢָ롣ֲ֧ʵĻֵοCameraParams.hк궨壬MV_CAML_BAUDRATE_9600  0x00000001
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ýӿ֧豸δʱá
 
 *  @~english
 *  @brief  Returns supported baudrates of the combined device and host interface
 *  @param  handle                      [IN]            Device handle
 *  @param  pnBaudrateAblity            [OUT]           Return pointer of the supported baudrates to user. 'OR' operation results of the supported baudrates. Refer to the 'CameraParams.h' for single value definitions, for example, MV_CAML_BAUDRATE_9600  0x00000001
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks This API is supported only by CameraLink device.\n
             This API support calls when devices are not connected.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CAML_GetSupportBaudrates(IN void* handle,unsigned int* pnBaudrateAblity);

/********************************************************************//**
 *  @~chinese
 *  @brief  ôڲȴʱ
 *  @param  handle                      [IN]            豸
 *  @param  nMillisec                   [IN]            ڲĵȴʱ, λΪms
 *  @return ɹ,MV_OK,ʧ,ش

 *  @~english
 *  @brief  Sets the timeout for operations on the serial port
 *  @param  handle                      [IN]            Device handle
 *  @param  nMillisec                   [IN]            Timeout in [ms] for operations on the serial port.
 *  @return Success, return MV_OK. Failure, return error code 
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CAML_SetGenCPTimeOut(IN void* handle, unsigned int nMillisec);


/************************************************************************/
/* U3V 豸еĽӿ                                            */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  U3VĴС
 *  @param  handle                      [IN]            豸
 *  @param  nTransferSize               [IN]            İС, ByteĬΪ1Mrang>=0x10000
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ӴСʵȡʱCPUռʡͬPCͲͬUSBչڲͬļԣòùܻȡͼķա
 
 *  @~english
 *  @brief  Set transfer size of U3V device
 *  @param  handle                      [IN]            Device handle
 *  @param  nTransferSize               [IN]            Transfer sizeBytedefault1Mrang>=0x10000
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Increasing the transmission packet size can reduce the CPU utilization at the time of fetching. However, different PCS and different USB extension CARDS have different compatibility, and if this parameter is set too large, there may be the risk of not getting the image.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_USB_SetTransferSize(IN void* handle, unsigned int nTransferSize);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡU3VĴС
 *  @param  handle                      [IN]            豸
 *  @param  pnTransferSize              [OUT]           İСָ, Byte
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ýӿڻȡǰU3VСĬ1M
 
 *  @~english
 *  @brief  Get transfer size of U3V device
 *  @param  handle                      [IN]            Device handle
 *  @param  pnTransferSize              [OUT]           Transfer sizeByte
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks This interface is used to get the current U3V transfer packet size, default 1M.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_USB_GetTransferSize(IN void* handle, unsigned int* pnTransferSize);

/********************************************************************//**
 *  @~chinese
 *  @brief  U3VĴͨ
 *  @param  handle                      [IN]            豸
 *  @param  nTransferWays               [IN]            ͨΧ1-10
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ûԸPCܡ豸ͼ֡ʡͼСڴʹʵضԸòеڡͬPCͲͬUSBչڲͬļԡ
 
 *  @~english
 *  @brief  Set transfer ways of U3V device
 *  @param  handle                      [IN]            Device handle
 *  @param  nTransferWays               [IN]            Transfer waysrang1-10
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks Users can adjust this parameter according to PC performance, camera image frame rate, image size, memory utilization and other factors. But different PCS and different USB expansion CARDS have different compatibility.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_USB_SetTransferWays(IN void* handle, unsigned int nTransferWays);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡU3VĴͨ
 *  @param  handle                      [IN]            豸
 *  @param  pnTransferWays              [OUT]           ָͨ
 *  @return ɹ,MV_OK,ʧ,ش
 *  @remarks ýӿڻȡǰU3V첽ȡڵ2000W豸MONO8ĬΪ3YUVΪĬ2RGBΪĬ1Ĭ8ڵ㡣
 
 *  @~english
 *  @brief  Get transfer ways of U3V device
 *  @param  handle                      [IN]            Device handle
 *  @param  pnTransferWays              [OUT]           Transfer ways
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks This interface is used to get the current number of U3V asynchronous feed nodes. For 2000W camera, MONO8 defaults to 3, YUV defaults to 2, RGB defaults to 1, and other cases default to 8 nodes.
************************************************************************/
MV_CAMCTRL_API int __stdcall MV_USB_GetTransferWays(IN void* handle, unsigned int* pnTransferWays);

/************************************************************************/
/* GenTLؽӿڣӿڿԸãֽӿڲ֧֣                    */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  ͨGenTLöInterfaces
 *  @param  pstIFList                   [OUT]           Interfacesб
 *  @param  strGenTLPath                [IN]            GenTLctiļ·
 *  @return ɹMV_OK󣬷ش 
 *  @remarks InterfacesбڴSDKڲģ̵߳øýӿʱ豸бڴͷź룬\n
             龡߳öٲ
 
 *  @~english
 *  @brief  Enumerate Interfaces with GenTL
 *  @param  pstIFList                   [OUT]           Interfaces List
 *  @param  strGenTLPath                [IN]            GenTL cti file path
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks The memory of the Interfaces list is allocated within the SDK. When the interface is invoked by multiple threads, the memory of the device list will be released and applied.\n
             It is recommended to avoid multithreaded enumeration operations as much as possible.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_EnumInterfacesByGenTL(IN OUT MV_GENTL_IF_INFO_LIST* pstIFList, IN const char * strGenTLPath);

/********************************************************************//**
 *  @~chinese
 *  @brief  ͨGenTL Interfaceö豸
 *  @param  pstIFInfo                   [IN]            InterfaceϢ
 *  @param  pstDevList                  [OUT]           豸б
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸бڴSDKڲģ̵߳øýӿʱ豸бڴͷź룬\n
             龡߳öٲ
 
 *  @~english
 *  @brief  Enumerate Devices with GenTL interface
 *  @param  pstIFInfo                   [IN]            Interface information
 *  @param  pstDevList                  [OUT]           Device List
 *  @return Success, return MV_OK. Failure, return error code 
 *  @remarks The memory of the list is allocated within the SDK. When the interface is invoked by multiple threads, the memory of the device list will be released and applied.\n
             It is recommended to avoid multithreaded enumeration operations as much as possible.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_EnumDevicesByGenTL(IN MV_GENTL_IF_INFO* pstIFInfo, IN OUT MV_GENTL_DEV_INFO_LIST* pstDevList);

/********************************************************************//**
 *  @~chinese
 *  @brief  ͨGenTL豸Ϣ豸
 *  @param  handle                      [OUT]           豸
 *  @param  pstDevInfo                  [IN]            豸Ϣṹָ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 豸ϢڲԴͳʼڲģ顣
 
 *  @~english
 *  @brief  Create Device Handle with GenTL Device Info
 *  @param  handle                      [OUT]           Device handle
 *  @param  pstDevInfo                  [IN]            Device Information
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Create required resources within library and initialize internal module according to input device information.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_CreateHandleByGenTL(OUT void ** handle, IN const MV_GENTL_DEV_INFO* pstDevInfo);


/************************************************************************/
/* XML                                                      */
/* XML parse tree generation                                            */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  ȡ豸XML
 *  @param  handle                      [IN]            豸
 *  @param  pData                       [OUT]           XMLݽջ
 *  @param  nDataSize                   [IN]            ջС
 *  @param  pnDataLen                   [OUT]           ʵݴС
 *  @return ɹMV_OK󣬷ش
 *  @remarks pDataΪNULLnDataSizeʵʵxmlļСʱݣpnDataLenxmlļС\n
             pDataΪЧַһ㹻ʱݱڸû棬pnDataLenxmlļʵʴС
 
 *  @~english
 *  @brief  Get camera feature tree XML
 *  @param  handle                      [IN]            Device handle
 *  @param  pData                       [OUT]           XML data receiving buffer
 *  @param  nDataSize                   [IN]            Buffer size
 *  @param  pnDataLen                   [OUT]           Actual data length
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks When pData is NULL or nDataSize than the actual XML file hours, do not copy the data, returned by pnDataLen XML file size.\n
             When pData is a valid cache address and the cache is large enough, copy the full data into the cache, and pnDataLen returns the actual size of the XML file.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_XML_GetGenICamXML(IN void* handle, IN OUT unsigned char* pData, IN unsigned int nDataSize, OUT unsigned int* pnDataLen);

/********************************************************************//**
 *  @~chinese
 *  @brief  õǰڵķģʽ
 *  @param  handle                      [IN]            豸
 *  @param  strName                     [IN]            ڵ
 *  @param  penAccessMode               [OUT]           ڵķģʽ
 *  @return ɹMV_OK󣬷ش

 *  @~english
 *  @brief  Get Access mode of cur node
 *  @param  handle                      [IN]            Device handle
 *  @param  strName                     [IN]            Name of node
 *  @param  penAccessMode               [OUT]           Access mode of the node
 *  @return Success, return MV_OK. Failure, return error code
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_XML_GetNodeAccessMode(IN void* handle, IN const char * strName, OUT MV_XML_AccessMode *penAccessMode);

/********************************************************************//**
 *  @~chinese
 *  @brief  õǰڵ
 *  @param  handle                      [IN]            豸
 *  @param  strName                     [IN]            ڵ
 *  @param  penInterfaceType            [OUT]           ڵ
 *  @return ɹMV_OK󣬷ش
 *  @remarks ýӿڿڵܽӿ֮ǰǰ֪ڵͣûѡʵܽӿڽнڵֵúͻȡ

 *  @~english
 *  @brief  Get Interface Type of cur node
 *  @param  handle                      [IN]            Device handle
 *  @param  strName                     [IN]            Name of node
 *  @param  penInterfaceType            [OUT]           Interface Type of the node
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks The interface can know the node type in advance before calling the universal interface, so as to facilitate users to select the appropriate universal interface for setting and obtaining the node value.
 ***********************************************************************/
MV_CAMCTRL_API int __stdcall MV_XML_GetNodeInterfaceType(IN void* handle, IN const char * strName, OUT MV_XML_InterfaceType *penInterfaceType);

/************************************************************************/
/* ӽӿ                                                               */
/* Additional interface                                                 */
/************************************************************************/
/********************************************************************//**
 *  @~chinese
 *  @brief  ͼƬ֧BmpJpeg.
 *  @param  handle                      [IN]            豸
 *  @param  pstSaveParam                [IN][OUT]       ͼƬṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ͨӿڿԽ豸ɼԭʼͼתJPEGBMPȸʽָڴУȻûԽתֱ֮ӱͼƬļýӿڵ޽ӿ˳ҪͼԴݾͿԽתȵMV_CC_GetOneFrameTimeoutMV_CC_RegisterImageCallBackExûصȡһ֡ͼݣȻͨýӿתʽ \n
             MV_CC_SaveImageEx2MV_CC_SaveImageExӲhandleΪ˱֤ӿڵͳһ
 
 *  @~english
 *  @brief  Save image, support Bmp and Jpeg.
 *  @param  handle                      [IN]            Device handle
 *  @param  pstSaveParam                [IN][OUT]       Save image parameters structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Once there is image data, you can call this API to convert the data. \n
             You can also call MV_CC_GetOneFrameTimeout or MV_CC_RegisterImageCallBackEx or MV_CC_GetImageBuffer to get one image frame and set the callback function, and then call this API to convert the format. \n
             Comparing with the API MV_CC_SaveImageEx, this API added the parameter handle to ensure the unity with other API. 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SaveImageEx2(IN void* handle, MV_SAVE_IMAGE_PARAM_EX* pstSaveParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  ͼļ
 *  @param  handle                      [IN]            豸
 *  @param  pstSaveFileParam            [IN][OUT]       ͼƬļṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿ֧BMP/JPEG/PNG/TIFF
 
 *  @~english
 *  @brief  Save the image file.
 *  @param  handle                      [IN]            Device handle
 *  @param  pstSaveFileParam            [IN][OUT]       Save the image file parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API support BMP/JPEG/PNG/TIFF.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SaveImageToFile(IN void* handle, MV_SAVE_IMG_TO_FILE_PARAM* pstSaveFileParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  3Dݣ֧PLYCSVOBJָʽ
 *  @param  handle                      [IN]            豸
 *  @param  pstPointDataParam           [IN][OUT]       ݲṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks 3Dݸʽ3Dļʽ֧PLY/CSV/OBJ/n
             Ŀǰ֧PixelType_Gvsp_Coord3D_ABC32PixelType_Gvsp_Coord3D_ABC32fPixelType_Gvsp_Coord3D_AB32PixelType_Gvsp_Coord3D_AB32fPixelType_Gvsp_Coord3D_AC32PixelType_Gvsp_Coord3D_AC32f,ݲ֧3Dʽ
 
 *  @~english
 *  @brief  Save 3D point data, support PLYCSV and OBJ
 *  @param  handle                      [IN]            Device handle
 *  @param  pstPointDataParam           [IN][OUT]       Save 3D point data parameters structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Save the 3D data format to 3D file formatsupport PLYCSV and OBJ/n
             only support PixelType_Gvsp_Coord3D_ABC32PixelType_Gvsp_Coord3D_ABC32fPixelType_Gvsp_Coord3D_AB32PixelType_Gvsp_Coord3D_AB32fPixelType_Gvsp_Coord3D_AC32PixelType_Gvsp_Coord3D_AC32f,Other 3D format is not supported now.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SavePointCloudData(IN void* handle, MV_SAVE_POINT_CLOUD_PARAM* pstPointDataParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  ͼת
 *  @param  handle                      [IN]            豸
 *  @param  pstRotateParam              [IN][OUT]       ͼתṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿֻ֧MONO8/RGB24/BGR24ʽݵ90/180/270ת
 
 *  @~english
 *  @brief  Rotate Image
 *  @param  handle                      [IN]            Device handle
 *  @param  pstRotateParam              [IN][OUT]       Rotate image parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API only support 90/180/270 rotation of data in the MONO8/RGB24/BGR24 format.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_RotateImage(IN void* handle, IN OUT MV_CC_ROTATE_IMAGE_PARAM* pstRotateParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  ͼת
 *  @param  handle                      [IN]            豸
 *  @param  pstFlipParam                [IN][OUT]       ͼתṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿֻ֧MONO8/RGB24/BGR24ʽݵĴֱˮƽת
 
 *  @~english
 *  @brief  Flip Image
 *  @param  handle                      [IN]            Device handle
 *  @param  pstFlipParam                [IN][OUT]       Flip image parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API only support vertical and horizontal reverse of data in the MONO8/RGB24/BGR24 format.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_FlipImage(IN void* handle, IN OUT MV_CC_FLIP_IMAGE_PARAM* pstFlipParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  ظʽת
 *  @param  handle                      [IN]            豸
 *  @param  pstCvtParam                 [IN][OUT]       ظʽתṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ͨӿڿԽ豸ɼԭʼͼתûظʽָڴСýӿڵ޽ӿ˳ҪͼԴݾͿԽתȵMV_CC_GetOneFrameTimeoutMV_CC_RegisterImageCallBackExûصȡһ֡ͼݣȻͨýӿתʽ豸ǰɼͼJPEGѹĸʽֵ֧øýӿڽת \n
 
 *  @~english
 *  @brief  Pixel format conversion
 *  @param  handle                      [IN]            Device handle
 *  @param  pstCvtParam                 [IN][OUT]       Convert Pixel Type parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API is used to transform the collected original data to pixel format and save to specified memory. There is no order requirement to call this API, the transformation will execute when there is image data. First call MV_CC_GetOneFrameTimeout or MV_CC_RegisterImageCallBackEx to set callback function, and get a frame of image data, then call this API to transform the format. \n
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_ConvertPixelType(IN void* handle, IN OUT MV_CC_PIXEL_CONVERT_PARAM* pstCvtParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  òֵ㷨
 *  @param  handle                      [IN]            豸
 *  @param  nBayerCvtQuality            [IN]            BayerĲֵ  0- 1- 2-ţĬΪţ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ڲͼתӿڵıֵMV_CC_ConvertPixelTypeMV_CC_SaveImageEx2ӿڲʹõĲֵ㷨Ǹýӿ趨ġ
 
 *  @~english
 *  @brief  Interpolation algorithm type setting
 *  @param  handle                      [IN]            Device handle
 *  @param  nBayerCvtQuality            [IN]            Bayer interpolation method  0-Fast 1-Equilibrium 2-Optimal
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Set the bell interpolation quality parameters of the internal image conversion interface, and the interpolation algorithm used in the MV CC ConvertPixelType and MV CC SaveImageEx2 interfaces is set by this interface.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetBayerCvtQuality(IN void* handle, IN unsigned int nBayerCvtQuality);

/********************************************************************//**
 *  @~chinese
 *  @brief  BayerʽֵGammaֵ
 *  @param  handle                      [IN]            豸
 *  @param  fBayerGammaValue            [IN]            Gammaֵ[0.1,4.0]
 *  @return ɹMV_OK󣬷ش 
 *  @remarks øֵڵMV_CC_ConvertPixelTypeMV_CC_SaveImageEx2ӿڽBayerʽתRGB24/BGR24/RGBA32/BGRA32ʱЧ
 
 *  @~english
 *  @brief  Set Gamma value after Bayer interpolation
 *  @param  handle                      [IN]            Device handle
 *  @param  fBayerGammaValue            [IN]            Gamma value[0.1,4.0]
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After setting the value, it work in the calling MV_CC_ConvertPixelType\MV_CC_SaveImageEx2 API convert Bayer to RGB24/BGR24/RGBA32/BGRA32.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetBayerGammaValue(IN void* handle, IN float fBayerGammaValue);

/********************************************************************//**
 *  @~chinese
 *  @brief  BayerʽֵɫУʹܺ;
 *  @param  handle                      [IN]            豸
 *  @param  pstCCMParam                 [IN]            ɫУṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ɫУɫУڵMV_CC_ConvertPixelTypeMV_CC_SaveImageEx2ӿڽBayerʽתRGB24/BGR24/RGBA32/BGRA32ʱЧ
 
 *  @~english
 *  @brief  Set CCM param after Bayer interpolation
 *  @param  handle                      [IN]            Device handle
 *  @param  pstCCMParam                 [IN]            Color correction parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks After enable the color correction and set the color correction matrix, It work in the calling MV_CC_ConvertPixelType\MV_CC_SaveImageEx2 API convert Bayer to RGB24/BGR24/RGBA32/BGRA32.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_SetBayerCCMParam(IN void* handle, IN MV_CC_CCM_PARAM* pstCCMParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  Bayer
 *  @param  handle                      [IN]            豸
 *  @param  pstNoiseEstimateParam       [IN][OUT]       BayerƲṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿֻ֧Bayer8/Bayer10/Bayer12ʽ,BayerʽתBayer8/Bayer10/Bayer12ʽ\n
             ýӿֻڴ˾ضɫſʹãϿߵߺ󣬼ʹøýӿڻᱨ
 
 *  @~english
 *  @brief  Noise estimate of Bayer format
 *  @param  handle                      [IN]            Device handle
 *  @param  pstNoiseEstimateParam       [IN][OUT]       Noise estimate parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API only support Bayer8/Bayer10/Bayer12 format, other Bayer format must Convert to Bayer8/Bayer10/Bayer12 format.\n
             This API is only available when the camera is turned on, and when the camera is disconnected or disconnected, continuing to use This API will return an error.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_BayerNoiseEstimate(IN void* handle, IN OUT MV_CC_BAYER_NOISE_ESTIMATE_PARAM* pstNoiseEstimateParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  Bayer
 *  @param  handle                      [IN]            豸
 *  @param  pstSpatialDenoiseParam      [IN][OUT]       Bayerṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ýӿֻ֧Bayer8/Bayer10/Bayer12ʽ,BayerʽתBayer8/Bayer10/Bayer12ʽ\n
             ýӿֻڴ˾ضɫſʹãϿߵߺ󣬼ʹøýӿڻᱨ
 
 *  @~english
 *  @brief  Spatial Denoise of Bayer format
 *  @param  handle                      [IN]            Device handle
 *  @param  pstSpatialDenoiseParam      [IN][OUT]       Spatial Denoise parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks This API only support Bayer8/Bayer10/Bayer12 format, other Bayer format must Convert to Bayer8/Bayer10/Bayer12 format.\n
             This API is only available when the camera is turned on, and when the camera is disconnected or disconnected, continuing to use This API will return an error.
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_BayerSpatialDenoise(IN void* handle, IN OUT MV_CC_BAYER_SPATIAL_DENOISE_PARAM* pstSpatialDenoiseParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  
 *  @param  handle                      [IN]            豸
 *  @param  pstDecodeParam              [IN][OUT]       ṹ
 *  @return ɹMV_OK󣬷ش 
 *  @remarks ȡѹݣͬʱֽ֧ǰʵʱͼˮӡϢǵǰ߲ʵʱȡģˮӡ쳣
 
 *  @~english
 *  @brief  High Bandwidth Decode
 *  @param  handle                      [IN]            Device handle
 *  @param  pstDecodeParam              [IN][OUT]       High Bandwidth Decode parameter structure
 *  @return Success, return MV_OK. Failure, return error code
 *  @remarks Decode the lossless compressed data from the camera into raw data 
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_HB_Decode(IN void* handle, IN OUT MV_CC_HB_DECODE_PARAM* pstDecodeParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸
 *  @param  handle                      [IN]            豸
 *  @param  strFileName                 [IN]            ļ
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Save camera feature
 *  @param  handle                      [IN]            Device handle
 *  @param  strFileName                 [IN]            File name
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_FeatureSave(IN void* handle, IN const char* strFileName);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸
 *  @param  handle                      [IN]            豸
 *  @param  strFileName                 [IN]            ļ
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Load camera feature
 *  @param  handle                      [IN]            Device handle
 *  @param  strFileName                 [IN]            File name
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_FeatureLoad(IN void* handle, IN const char* strFileName);

/********************************************************************//**
 *  @~chinese
 *  @brief  豸ȡļ
 *  @param  handle                      [IN]            豸
 *  @param  pstFileAccess               [IN]            ļȡṹ
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Read the file from the camera
 *  @param  handle                      [IN]            Device handle
 *  @param  pstFileAccess               [IN]            File access structure
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_FileAccessRead(IN void* handle, IN MV_CC_FILE_ACCESS * pstFileAccess);

/********************************************************************//**
 *  @~chinese
 *  @brief  ļд豸
 *  @param  handle                      [IN]            豸
 *  @param  pstFileAccess               [IN]            ļȡṹ
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Write the file to camera
 *  @param  handle                      [IN]            Device handle
 *  @param  pstFileAccess               [IN]            File access structure
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_FileAccessWrite(IN void* handle, IN MV_CC_FILE_ACCESS * pstFileAccess);

/********************************************************************//**
 *  @~chinese
 *  @brief  ȡļȡĽ
 *  @param  handle                      [IN]            豸
 *  @param  pstFileAccessProgress       [IN]            
 *  @return ɹMV_OK󣬷ش ǰļȡ״̬
 
 *  @~english
 *  @brief  Get File Access Progress 
 *  @param  handle                      [IN]            Device handle
 *  @param  pstFileAccessProgress       [IN]            File access Progress
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_GetFileAccessProgress(IN void* handle, OUT MV_CC_FILE_ACCESS_PROGRESS * pstFileAccessProgress);

/********************************************************************//**
 *  @~chinese
 *  @brief  ʼ¼
 *  @param  handle                      [IN]            豸
 *  @param  pstRecordParam              [IN]            ¼ṹ
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Start Record
 *  @param  handle                      [IN]            Device handle
 *  @param  pstRecordParam              [IN]            Record param structure
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_StartRecord(IN void* handle, IN MV_CC_RECORD_PARAM* pstRecordParam);

/********************************************************************//**
 *  @~chinese
 *  @brief  ¼
 *  @param  handle                      [IN]            豸
 *  @param  pstInputFrameInfo           [IN]            ¼ݽṹ
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Input RAW data to Record
 *  @param  handle                      [IN]            Device handle
 *  @param  pstInputFrameInfo           [IN]            Record data structure
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_InputOneFrame(IN void* handle, IN MV_CC_INPUT_FRAME_INFO * pstInputFrameInfo);

/********************************************************************//**
 *  @~chinese
 *  @brief  ֹͣ¼
 *  @param  handle                      [IN]            豸
 *  @return ɹMV_OK󣬷ش 
 
 *  @~english
 *  @brief  Stop Record
 *  @param  handle                      [IN]            Device handle
 *  @return Success, return MV_OK. Failure, return error code
 ************************************************************************/
MV_CAMCTRL_API int __stdcall MV_CC_StopRecord(IN void* handle);
#ifdef __cplusplus
}
#endif 

#endif //_MV_CAMERA_CTRL_H_
