#include "exportData.h"
#include "QtCore\qfile.h"
#include "QtCore\qtextstream.h"
#include<string>

Cigarette* cg;
extern SyncQueue<_ExportDataInfo>* export_Data_Info_queue;
extern ConfPath g_conf_path;

ExportDataThread::ExportDataThread(QObject* parent) : QThread(parent)
{
	//char xmlPath[256];
	cg->read_conf(g_conf_path);

	for (int index = 0; index < NumberOfSupportedCameras; index++)
	{
		XMLError error;
		pDocument[index] = new tinyxml2::XMLDocument();
		//memset(xmlPath, 0, 256);
		QString xmlPath = QString(EXPORTDATA_FILE).arg(index);
		//sprintf(xmlPath, EXPORTDATA_FILE, index);
		QString filePath = g_conf_path.config_path + "/" + xmlPath;

		error = pDocument[index]->LoadFile(filePath.toLocal8Bit().constData());
		if (error != XML_SUCCESS)
		{
			XMLDeclaration* declaration = pDocument[index]->NewDeclaration();
			pDocument[index]->InsertFirstChild(declaration);
			XMLElement* root = pDocument[0]->NewElement("Root");
			pDocument[0]->InsertEndChild(root);
			pDocument[index]->SaveFile(filePath.toLocal8Bit().constData());
		}
		/*
		_ExportDataInfo data;
		data.cameraId = index;
		data.cameraTotal = 3;
		data.timeCost = "3ms";
		insertXMLNode(xmlPath, data);
		*/

		_XMLExportDataInfo data;
		data.cameraId = index;
		data.cameraTotal = 3;
		data.TotalCheckNum = 1000;
		data.TotalKickNum = 900;
		data.AcquisitionSpeed = "3ms";
		insertXMLNode(filePath.toLocal8Bit().constData(), data);
	}
}

void ExportDataThread::init()
{
	b_quit = false;
}

void ExportDataThread::start_work()
{
	start(HighestPriority);
}

void ExportDataThread::stop()
{
	b_quit = true;
	_ExportDataInfo data;
	export_Data_Info_queue->put(data);
}

bool _ExportDataInfo::getAverageData(map<string, float> &averageData, int index)
{
	tinyxml2::XMLDocument doc;
	//char xmlPath[256];
	XMLError error;
	map<string, float> data;
	//memset(xmlPath, 0, 256);
	QString xmlPath = QString(EXPORTDATA_FILE).arg(index);
	//sprintf(xmlPath, EXPORTDATA_FILE, index);
	QString filePath = g_conf_path.config_path + "/" + xmlPath;
	error = doc.LoadFile(filePath.toLocal8Bit().constData());
	if (error != XML_SUCCESS)
		if (doc.LoadFile(filePath.toLocal8Bit().constData()) != 0)
		{
			cout << "load xml file failed" << endl;
			return false;
		}
	XMLElement* root = doc.RootElement();
	XMLElement* userNode = root->FirstChildElement("Camera");
	data["CameraId"] = index;
	data["IsNG"] = 0;
	data["IsJdExist"] = 0;
	data["Total"] = 0;
	while (userNode != NULL)
	{
		if (atoi(userNode->Attribute("Id")) == index) {
			data["IsNG"]++;
			data["IsJdExist"]++;
			data["Total"]++;
			XMLElement* IsNgNode = userNode->FirstChildElement("IsNG");
			data["IsNg"] = strcmp(IsNgNode->GetText(), "TRUE");
			XMLElement* IsJdExistNode = userNode->FirstChildElement("IsJdExist");
			if (strcmp(IsJdExistNode->GetText(), "TRUE") == 0)
				data["IsJdExist"]++;
			XMLElement* TimeCostNode = userNode->FirstChildElement("TimeCost");
			data["TimeCost"] += stof(TimeCostNode->GetText());
		}
		userNode = userNode->NextSiblingElement();//һֵܽڵ
	}
	if (data["Total"] == 0)
		return false;
	data["TimeCost"] = data["TimeCost"] / data["Total"];

	averageData = data;

	return true;
}

int ExportDataThread::insertXMLNode(const char* xmlPath, _ExportDataInfo& data)
{
	XMLElement* root = pDocument[data.cameraId]->RootElement();

	if (root == NULL) {
		root = pDocument[data.cameraId]->NewElement("Root");
		pDocument[data.cameraId]->InsertEndChild(root);
		pDocument[data.cameraId]->SaveFile(xmlPath);
	}

	XMLElement* CameraNode = pDocument[data.cameraId]->NewElement("Camera");
	CameraNode->SetAttribute("Id", data.cameraId);
	CameraNode->SetAttribute("Count ", data.cameraTotal);
	root->InsertEndChild(CameraNode);

	XMLElement* IsNG = pDocument[data.cameraId]->NewElement("IsNG");
	XMLText* IsNGText = pDocument[data.cameraId]->NewText(data.isNg ? "TRUE" : "FALSE");
	IsNG->InsertEndChild(IsNGText);
	CameraNode->InsertEndChild(IsNG);

	XMLElement* timeCost = pDocument[data.cameraId]->NewElement("TimeCost");
	timeCost->InsertEndChild(pDocument[data.cameraId]->NewText(data.timeCost.c_str()));
	CameraNode->InsertEndChild(timeCost);

	XMLElement* isJdExist = pDocument[data.cameraId]->NewElement("IsJdExist");
	isJdExist->InsertEndChild(pDocument[data.cameraId]->NewText(data.isJdExist[0] ? "TRUE" : "FALSE"));
	CameraNode->InsertEndChild(isJdExist);

	// XMLElement* jdInterval = pDocument[data.cameraId]->NewElement("JdInterval");
	// jdInterval->InsertEndChild(pDocument[data.cameraId]->NewText((const char*)&data.jdInterval));
	// CameraNode->InsertEndChild(jdInterval);

	// XMLElement* jdInterval = pDocument[data.cameraId]->NewElement("jdPointsLocation");
	// jdInterval->InsertEndChild(pDocument[data.cameraId]->NewText(data.getPoint(0).c_str()));
	// CameraNode->InsertEndChild(jdInterval);

	return pDocument[data.cameraId]->SaveFile(xmlPath);
}

int ExportDataThread::insertXMLNode(const char* xmlPath, _XMLExportDataInfo& data)
{
	XMLElement* root = pDocument[data.cameraId]->RootElement();

	if (root == NULL) {
		root = pDocument[data.cameraId]->NewElement("Root");
		pDocument[data.cameraId]->InsertEndChild(root);
		pDocument[data.cameraId]->SaveFile(xmlPath);
	}

	XMLElement* CameraNode = pDocument[data.cameraId]->NewElement("Camera");
	CameraNode->SetAttribute("Id", data.cameraId);
	CameraNode->SetAttribute("Count ", data.cameraTotal);
	root->InsertEndChild(CameraNode);

	//XMLElement* IsNG = pDocument[data.cameraId]->NewElement("IsNG");
	//XMLText* IsNGText = pDocument[data.cameraId]->NewText(data.isNg ? "TRUE" : "FALSE");
	//IsNG->InsertEndChild(IsNGText);
	//CameraNode->InsertEndChild(IsNG);

	//XMLElement* timeCost = pDocument[data.cameraId]->NewElement("TimeCost");
	//timeCost->InsertEndChild(pDocument[data.cameraId]->NewText(data.timeCost.c_str()));
	//CameraNode->InsertEndChild(timeCost);

	XMLElement* TotalCheckNum = pDocument[data.cameraId]->NewElement("TotalCheckNum");
	std::string temp_str = std::to_string(data.TotalCheckNum);
	const char* temp_char = temp_str.c_str();
	TotalCheckNum->InsertEndChild(pDocument[data.cameraId]->NewText(temp_char));
	CameraNode->InsertEndChild(TotalCheckNum);

	XMLElement* TotalKickNum = pDocument[data.cameraId]->NewElement("TotalKickNum");
	temp_str = std::to_string(data.TotalKickNum);
	temp_char = temp_str.c_str();
	TotalKickNum->InsertEndChild(pDocument[data.cameraId]->NewText(temp_char));
	CameraNode->InsertEndChild(TotalKickNum);

	XMLElement* AcquisitionSpeed = pDocument[data.cameraId]->NewElement("AcquisitionSpeed");
	AcquisitionSpeed->InsertEndChild(pDocument[data.cameraId]->NewText(data.AcquisitionSpeed.c_str()));
	CameraNode->InsertEndChild(AcquisitionSpeed);

	return pDocument[data.cameraId]->SaveFile(xmlPath);
}

void ExportDataThread::run()
{
	while (!b_quit) {
		_ExportDataInfo element;
		export_Data_Info_queue->take(element);
		if (element.cameraId != -1) {
			char buf[256];
			memset(buf, 0, 256);
			QString xmlPath = QString(EXPORTDATA_FILE).arg(element.cameraId);
			QString filePath = g_conf_path.config_path + "/" + xmlPath;
			sprintf(buf, filePath.toLocal8Bit().constData(), element.cameraId);
			insertXMLNode(buf, element);
		}
	}

	for (int index = 0; index < NumberOfSupportedCameras; index++) {
		pDocument[index]->~XMLDocument();
	}
	
}

void ExportDataThread::check_save_dir(std::string dir_path)
{
	bool b_find = false;
	for (int i = 0; i < vec_save_dirs.size(); i++)
	{
		if (dir_path == vec_save_dirs[i])
		{
			b_find = true;
		}
	}
	if (!b_find)
	{
		//QString dir_str = QString::fromStdString(dir_path);
		QString dir_str = QString::fromLocal8Bit(QByteArray::fromRawData(dir_path.c_str(), dir_path.size()));

		QDir dir;
		if (!dir.exists(dir_str))
		{
			dir.mkpath(dir_str);
		}
		vec_save_dirs.push_back(dir_path);
	}
}